/* Main Calendar Widget
 * FIXME: for now this is just a wrapper around gnome-cal.[ch]
 * Copyright (C) 2002  Sebastian Rittau <srittau@jroger.in-berlin.de>
 *
 * $Id: calendar-widget.c,v 1.1.2.4 2002/05/27 18:22:54 srittau Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>

#include <glib.h>

#include <gnome.h>

#include "calendar-widget.h"
#include "gnome-cal.h"

static void calendar_widget_init	(CalendarWidget		*widget);
static void calendar_widget_finalize	(GObject		*object);
static void calendar_widget_setup	(CalendarWidget		*widget);
static void calendar_widget_class_init	(CalendarWidgetClass	*klass);


struct _CalendarWidgetPriv {
	GtkWidget *calendar;
};


static GtkVBoxClass *parent_class = NULL;

/*
 * Class and Instance Setup
 */

GType
calendar_widget_get_type (void)
{
	static GType calendar_widget_type = 0;

	if (!calendar_widget_type) {
		static const GTypeInfo calendar_widget_info = {
			sizeof (CalendarWidgetClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) calendar_widget_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (CalendarWidget),
			1,    /* n_preallocs */
			(GInstanceInitFunc) calendar_widget_init,
		};

		calendar_widget_type = g_type_register_static (GTK_TYPE_VBOX,
							       "CalendarWidget",
							       &calendar_widget_info,
							       0);
	}

	return calendar_widget_type;
}

static void
calendar_widget_class_init (CalendarWidgetClass *klass)
{
	GObjectClass *gobject_class;

	g_return_if_fail (klass != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET_CLASS (klass));

	gobject_class = G_OBJECT_CLASS (klass);

	gobject_class->finalize = calendar_widget_finalize;

	parent_class = g_type_class_peek_parent (klass);
}

static void
calendar_widget_init (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	widget->priv = g_new0 (CalendarWidgetPriv, 1);

	calendar_widget_setup (widget);
}

static void
calendar_widget_finalize (GObject *object)
{
	CalendarWidget *widget;

	g_return_if_fail (object != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (object));

	widget = CALENDAR_WIDGET (object);

	g_free (widget->priv);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

/*
 * Setup
 */

static void
calendar_widget_setup (CalendarWidget *widget)
{
	CalendarWidgetPriv *priv;

	priv = widget->priv;

	priv->calendar = gnome_calendar_new ();
	gtk_container_add (GTK_CONTAINER (widget), priv->calendar);
	gtk_widget_show (priv->calendar);
}

/*
 * Public Methods
 */

GtkWidget *
calendar_widget_new (void)
{
	CalendarWidget *widget;

	widget = g_object_new (CALENDAR_TYPE_WIDGET, NULL);

	return GTK_WIDGET (widget);
}

/*
 * Save the calendar to FILE (or the current file name if FILE is NULL).
 */

/* FIXME: from main:*/
void save_calendar (GnomeCalendar *gcal);

gboolean
calendar_widget_save (CalendarWidget *widget, const gchar *file)
{
	CalendarWidgetPriv *priv;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);

	priv = widget->priv;

	if (file) {
		Calendar *calendar;

		calendar = GNOME_CALENDAR (priv->calendar)->cal;

		g_free (calendar->filename);
		calendar->filename = g_strdup (file);
	}

	save_calendar (GNOME_CALENDAR (priv->calendar));

	return TRUE;
}

void
calendar_widget_next (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_next (GNOME_CALENDAR (widget->priv->calendar));
}

void
calendar_widget_previous (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_previous (GNOME_CALENDAR (widget->priv->calendar));
}

void
calendar_widget_today (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	gnome_calendar_goto_today (GNOME_CALENDAR (widget->priv->calendar));
}

void
calendar_widget_goto (CalendarWidget *widget, struct tm *tm)
{
	time_t t;

	t = mktime (tm);

	/* FIXME: make gnome_calendar_goto accept dates outside the Unix range */
	if (t >= 0)
		gnome_calendar_goto (GNOME_CALENDAR (widget->priv->calendar), t);
}

gboolean
calendar_widget_set_calendar (CalendarWidget *widget,
			      const gchar *filename,
			      GError **error)
{
	CalendarWidgetPriv *priv;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);
	g_return_val_if_fail (filename != NULL, FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	priv = widget->priv;

	if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
		g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_NOENT,
			     _("Error while loading calendar file %s: %s!"), filename, _("file does not exist"));
		return FALSE;
	} else {
		if (!gnome_calendar_load (GNOME_CALENDAR (priv->calendar), filename)) {
			g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
				     _("Error while loading calendar file %s: %s!"), filename, _("unknown error"));
			return FALSE;
		}
	}

	return TRUE;
}

gboolean
calendar_widget_set_calendar_default (CalendarWidget *widget, GError **error)
{
	gchar *filename;
	gboolean ret;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	filename = g_build_filename (g_get_home_dir (),
				     ".gnome/user-cal.vcf",
				     NULL);

	ret = calendar_widget_set_calendar (widget, filename, error);

	g_free (filename);

	return ret;
}

void
calendar_widget_set_calendar_new (CalendarWidget *widget)
{
	g_return_if_fail (widget != NULL);
	g_return_if_fail (CALENDAR_IS_WIDGET (widget));

	/* FIXME: implement */
}

/* FIXME: from main.c */
void save_default_calendar (GnomeCalendar *gcal);
gboolean
calendar_widget_save_calendar (CalendarWidget *widget, GError **error)
{
	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), FALSE);
	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);

	save_default_calendar (GNOME_CALENDAR (widget->priv->calendar));

	return TRUE;
}

/* FIXME: obsolete */
GnomeCalendar *
calendar_widget_get_gncal (CalendarWidget *widget)
{
	g_return_val_if_fail (widget != NULL, NULL);
	g_return_val_if_fail (CALENDAR_IS_WIDGET (widget), NULL);

	return GNOME_CALENDAR (widget->priv->calendar);
}
