/* Day view widget for gncal
 *
 * Copyright (C) 1998 The Free Software Foundation
 *
 * Authors: Federico Mena   <quartic@gimp.org>
 *          Miguel de Icaza <miguel@kernel.org>
 */

#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <gnome.h>

#include "eventedit.h"
#include "gncal-day-view.h"
#include "gnomecal-config.h"
#include "popup-menu.h"
#include "timeutil.h"
#include "view-utils.h"


#define TEXT_BORDER 2
#define MIN_INFO_WIDTH 50


static void gncal_day_view_class_init   (GncalDayViewClass *class);
static void gncal_day_view_init         (GncalDayView      *dview);
static void gncal_day_view_destroy      (GtkObject         *object);
static void gncal_day_view_realize      (GtkWidget         *widget);
static void gncal_day_view_size_request (GtkWidget         *widget,
					 GtkRequisition    *requisition);
static gint gncal_day_view_expose       (GtkWidget         *widget,
					 GdkEventExpose    *event);
static gint gncal_day_view_button_press (GtkWidget         *widget,
					 GdkEventButton    *event);


static GtkWidgetClass *parent_class;


GType
gncal_day_view_get_type (void)
{
	static GType day_view_type = 0;

	if (!day_view_type) {
		static const GTypeInfo day_view_info = {
			sizeof (GncalDayViewClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) gncal_day_view_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (GncalDayView),
			0,    /* n_preallocs */
			(GInstanceInitFunc) gncal_day_view_init,
		};

		day_view_type = g_type_register_static (GTK_TYPE_WIDGET,
							"GncalDayView",
							&day_view_info,
							0);
	}

	return day_view_type;
}

static void
gncal_day_view_class_init (GncalDayViewClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	object_class = (GtkObjectClass *) class;
	widget_class = (GtkWidgetClass *) class;

	parent_class = gtk_type_class (gtk_widget_get_type ());

	object_class->destroy = gncal_day_view_destroy;

	widget_class->realize = gncal_day_view_realize;
	widget_class->size_request = gncal_day_view_size_request;
	widget_class->expose_event = gncal_day_view_expose;
	widget_class->button_press_event = gncal_day_view_button_press;
}

static void
gncal_day_view_init (GncalDayView *dview)
{
	GTK_WIDGET_UNSET_FLAGS (dview, GTK_NO_WINDOW);

	dview->calendar = NULL;

	dview->lower = 0;
	dview->upper = 0;

	dview->events = 0;
	dview->shadow_type = GTK_SHADOW_ETCHED_IN;
}

static void
gncal_day_view_destroy (GtkObject *object)
{
	GncalDayView *dview;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GNCAL_IS_DAY_VIEW (object));

	dview = GNCAL_DAY_VIEW (object);

	g_free (dview->day_str);
	dview->day_str = NULL;

	if (dview->events) {
		calendar_destroy_event_list (dview->events);
		dview->events = NULL;
	}

	GTK_OBJECT_CLASS (parent_class)->destroy (object);
}

static void
new_appointment (GtkWidget *widget, gpointer data)
{
	GConfClient *gconf;
	GncalDayView *dayview = GNCAL_DAY_VIEW (data);
	iCalObject *ico;
	GtkWidget *ee;
	struct tm tm;
	gint day_begin;

	gconf = gconf_client_get_default ();
	day_begin = gconf_client_get_int (gconf, CONFIG_CAL_DAY_START, NULL);
	g_object_unref (G_OBJECT (gconf));

	ico = ical_new_with_defaults ();
	ico->new = 1;

	tm = *localtime (&dayview->lower);
	tm.tm_hour = day_begin;
	ico->dtstart = mktime (&tm);
	tm.tm_hour++;
	ico->dtend   = mktime (&tm);
	ee = event_editor_new (dayview->calendar, ico);
	gtk_widget_show (ee);
}

static void
context_menu (GncalDayView *dayview, GdkEventButton *event)
{
	static struct menu_item main_items[] = {
		{ N_("New appointment..."), (GtkSignalFunc) new_appointment, NULL, TRUE }
	};

	main_items [0].data = dayview;
	popup_menu (main_items, 1, event);
}

static gint
gncal_day_view_button_press (GtkWidget *widget, GdkEventButton *event)
{
	GncalDayView *dayview;

	dayview = GNCAL_DAY_VIEW (widget);

	if (event->button == 3)
		context_menu (dayview, event);

	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS)
		gnome_calendar_dayjump (dayview->calendar, dayview->lower);

	return TRUE;
}

GtkWidget *
gncal_day_view_new (void)
{
	GncalDayView *dview;

	dview = g_object_new (GNCAL_TYPE_DAY_VIEW, NULL);

	return GTK_WIDGET (dview);
}

static void
gncal_day_view_realize (GtkWidget *widget)
{
	GdkWindowAttr attributes;
	gint attributes_mask;

	g_return_if_fail (widget != NULL);
	g_return_if_fail (GNCAL_IS_DAY_VIEW (widget));

	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.x = widget->allocation.x;
	attributes.y = widget->allocation.y;
	attributes.width = widget->allocation.width;
	attributes.height = widget->allocation.height;
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.visual = gtk_widget_get_visual (widget);
	attributes.colormap = gtk_widget_get_colormap (widget);
	attributes.event_mask = (gtk_widget_get_events (widget)
				 | GDK_EXPOSURE_MASK
				 | GDK_BUTTON_PRESS_MASK);

	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
	gdk_window_set_user_data (widget->window, widget);

	widget->style = gtk_style_attach (widget->style, widget->window);

	gdk_window_set_background (widget->window, &widget->style->bg[GTK_STATE_PRELIGHT]);
}

static void
gncal_day_view_size_request (GtkWidget *widget, GtkRequisition *requisition)
{
	GtkStyle *style;
	PangoLayout *layout;
	GncalDayView *dview;
	int str_height, str_width, width;

	g_return_if_fail (widget != NULL);
	g_return_if_fail (GNCAL_IS_DAY_VIEW (widget));
	g_return_if_fail (requisition != NULL);

	dview = GNCAL_DAY_VIEW (widget);

	/* border and min width */

	style     = gtk_widget_get_style (widget);

	layout = gtk_widget_create_pango_layout (widget, dview->day_str);
	pango_layout_get_pixel_size (layout, &str_width, &str_height);

	width = MAX (MIN_INFO_WIDTH, str_width);

	requisition->width  = 2 * (style->xthickness + TEXT_BORDER) + width;
	requisition->height = 2 * (style->ythickness + TEXT_BORDER);

	/* division line */

	requisition->height += 2 * TEXT_BORDER + style->ythickness;

	/* title and at least one line of text */

	requisition->height += 2 * str_height;
}

static gint
gncal_day_view_expose (GtkWidget *widget, GdkEventExpose *event)
{
	GncalDayView *dview;
	int x1, y1, width, height;
	GtkStyle *style;
	PangoLayout *layout;
	GdkRectangle rect, dest;
	GdkGC *gc;
	int str_width, str_height;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (GNCAL_IS_DAY_VIEW (widget), FALSE);
	g_return_val_if_fail (event != NULL, FALSE);

	if (!GTK_WIDGET_DRAWABLE (widget))
		return FALSE;

	dview = GNCAL_DAY_VIEW (widget);

	style  = gtk_widget_get_style (widget);
	layout = gtk_widget_create_pango_layout (widget, dview->day_str);

	gc = style->fg_gc [GTK_STATE_NORMAL];

	x1 = style->xthickness;
	y1 = style->ythickness;
	width  = widget->allocation.width - 2 * x1;
	height = widget->allocation.height - 2 * y1;

	/* Clear and paint frame shadow */

	gdk_window_clear_area (widget->window,
			       event->area.x, event->area.y,
			       event->area.width, event->area.height);

	gtk_paint_shadow (style, widget->window,
			  GTK_STATE_NORMAL, dview->shadow_type,
			  NULL, NULL, NULL,
			  0, 0,
			  widget->allocation.width,
			  widget->allocation.height);

	/* Clear and paint title */

	pango_layout_get_pixel_size (layout, &str_width, &str_height);

	rect.x = x1;
	rect.y = y1;
	rect.width = width;
	rect.height = 2 * TEXT_BORDER + str_height;

	if (gdk_rectangle_intersect (&rect, &event->area, &dest)) {
		gdk_draw_rectangle (widget->window,
				    widget->style->bg_gc[GTK_STATE_NORMAL],
				    TRUE,
				    dest.x, dest.y,
				    dest.width, dest.height);

		dest = rect;

		dest.x += TEXT_BORDER;
		dest.y += TEXT_BORDER;
		dest.width  -= 2 * TEXT_BORDER;
		dest.height -= 2 * TEXT_BORDER;

		gdk_gc_set_clip_rectangle (gc, &dest);

		pango_layout_get_pixel_size (layout, &str_width, NULL);

		gdk_draw_layout (widget->window, gc,
				 dest.x + (dest.width - str_width) / 2,
				 dest.y,
				 layout);

		gdk_gc_set_clip_rectangle (gc, NULL);
	}

	/* Division line */

	gtk_paint_hline (widget->style,
			 widget->window,
			 GTK_STATE_NORMAL,
			 NULL, NULL, NULL,
			 rect.x,
			 rect.x + rect.width - 1,
			 rect.y + rect.height);

	/* Text */

	if (dview->events != NULL){
		rect.x = x1 + TEXT_BORDER;
		rect.y = y1 + 3 * TEXT_BORDER + str_height +
			style->ythickness;
		rect.width = width - 2 * TEXT_BORDER;
		rect.height = height - (rect.y - y1) - TEXT_BORDER;
		
		if (gdk_rectangle_intersect (&rect, &event->area, &dest))
			view_utils_draw_events (
				widget, widget->window, gc,
				&rect,
				VIEW_UTILS_DRAW_END | VIEW_UTILS_DRAW_SPLIT,
				dview->events,
				dview->lower,
				dview->upper);
	}

	return FALSE;
}

void
gncal_day_view_update (GncalDayView *dview, iCalObject *ico, int flags)
{
	struct tm tm;
	gchar buf[256];

	g_return_if_fail (dview != NULL);
	g_return_if_fail (GNCAL_IS_DAY_VIEW (dview));
	g_return_if_fail (dview->calendar != NULL);
	g_return_if_fail (dview->calendar->cal != NULL);

	if (dview->day_str)
		g_free (dview->day_str);

	tm = *localtime (&dview->lower);
	strftime (buf, sizeof (buf), _("%A %d"), &tm);
	dview->day_str = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);

	if (dview->events)
		calendar_destroy_event_list (dview->events);

	dview->events = calendar_get_events_in_range (dview->calendar->cal,
						      dview->lower,
						      dview->upper);

	gtk_widget_queue_draw (GTK_WIDGET (dview));
}

void
gncal_day_view_set_bounds (GncalDayView *dview, time_t lower, time_t upper)
{
	g_return_if_fail (dview != NULL);
	g_return_if_fail (GNCAL_IS_DAY_VIEW (dview));

	if ((lower != dview->lower) || (upper != dview->upper)) {
		dview->lower = lower;
		dview->upper = upper;

		gncal_day_view_update (dview, NULL, 0);
	}
}

void
gncal_day_view_set_shadow (GncalDayView *dview, GtkShadowType shadow_type)
{
	g_return_if_fail (dview != NULL);
	g_return_if_fail (GNCAL_IS_DAY_VIEW (dview));

	if (shadow_type != dview->shadow_type) {
		dview->shadow_type = shadow_type;

		gtk_widget_queue_draw (GTK_WIDGET (dview));
	}
}
