/* General-purpose monthly calendar canvas item for GNOME
 *
 * Copyright (C) 1998 Red Hat Software, Inc.
 *
 * Author: Federico Mena <federico@nuclecu.unam.mx>
 *
 * Changed in 2002 by Sebastian Rittau <srittau@jroger.in-berlin.de> for
 * the GNOME 2 port.
 *
 * $Id: gnome-month-item.c,v 1.17.4.14 2002/06/02 00:19:40 srittau Exp $
 */

#include <config.h>

#include <string.h>
#include <math.h>
#include <time.h>

#include <glib.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <gnome.h>

#include "gnome-month-item.h"
#include "gnomecal-config.h"
#include "gnomecal-marshal.h"


#define DEFAULT_FONT "fixed"

#define NUMBER_OF_DAYS 42

struct _GnomeMonthItemPriv {
	guint year;			/* Year to show (full, no two-digit crap) */
	guint month;			/* Month to show (0-11) */
	guint day;			/* Currently selected day (1-31) */

	double x, y;			/* Position at anchor */
	double width, height;		/* Size of calendar */
	GtkAnchorType anchor;		/* Anchor side for calendar */

	double head_padding;		/* Padding to use between heading lines and text */
	double day_padding;		/* Padding to use between day number lines and text */

	char *day_names[7];		/* Names to use for the day labels, starting from Sunday */

	double head_height;		/* Height of the headings row */
	GtkAnchorType head_anchor;	/* Anchor side for the heading labels */
	GtkAnchorType day_anchor;	/* Anchor side for the day number labels */

	GnomeCanvasItem **items;	/* All the items that make up the calendar */
	int day_numbers[NUMBER_OF_DAYS];/* The numbers of the days, as they are shown in the display */

	PangoFontDescription *head_font;/* Font for the headings */
	PangoFontDescription *day_font;	/* Font for the day numbers */

	gchar *head_color;		/* Color for heading labels */
	gchar *outline_color;		/* Color for the outline (lines and heading boxes) */
	gchar *day_box_color;		/* Color for the day boxes */
	gchar *day_color;		/* Color for day number labels */

	gboolean start_on_monday : TRUE;/* Start the week on Monday? If FALSE start from Sunday */

	/* Signal IDs */

	guint notify1, notify2;
};


/* Number of days in a month, for normal and leap years */
static const int days_in_month[2][12] = {
	{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
	{ 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
};

/* The weird month of September 1752, where 3 Sep through 13 Sep were eliminated due to the
 * Gregorian reformation.
 */
static const int sept_1752[NUMBER_OF_DAYS] = {
	 0,  0,  1,  2, 14, 15, 16,
	17, 18, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 30,
	 0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0
};

#define REFORMATION_DAY 639787		/* First day of the reformation, counted from 1 Jan 1 */
#define MISSING_DAYS 11			/* They corrected out 11 days */
#define THURSDAY 4			/* First day of reformation */
#define SATURDAY 6			/* Offset value; 1 Jan 1 was a Saturday */
#define SEPT_1752_START 2		/* Start day within month */
#define SEPT_1752_END 20		/* End day within month */


enum {
	/* FIXME: temporary solution: This is emitted whenever we changed
	 * our looks on our own (e.g. because the configuration changed),
	 * so our users can take appropriate action to redraw parts of us
	 * that they have changed. Of course users should *not* be able
	 * to change us in any way, except through an explicit API.
	 */
	SIGNAL_REORDERED,
	SIGNAL_LAST
};

enum {
	ARG_YEAR = 1,
	ARG_MONTH,
	ARG_DAY,
	ARG_X,
	ARG_Y,
	ARG_WIDTH,
	ARG_HEIGHT,
	ARG_ANCHOR,
	ARG_HEAD_PADDING,
	ARG_DAY_PADDING,
	ARG_DAY_NAMES,
	ARG_HEADING_HEIGHT,
	ARG_HEADING_ANCHOR,
	ARG_DAY_ANCHOR,
	ARG_HEAD_FONT,
	ARG_HEAD_FONT_DESC,
	ARG_DAY_FONT,
	ARG_DAY_FONT_DESC
};


static void gnome_month_item_class_init    (GnomeMonthItemClass *class);
static void gnome_month_item_init          (GnomeMonthItem      *mitem);
static void gnome_month_item_finalize      (GObject             *object);
static void gnome_month_item_dispose       (GObject             *object);
static void gnome_month_item_set_property  (GObject             *object,
					    guint                property_id,
					    const GValue        *value,
					    GParamSpec          *pspec);
static void gnome_month_item_get_property  (GObject             *object,
					    guint                property_id,
					    GValue              *value,
					    GParamSpec          *pspec);
static gboolean gnome_month_item_day_event (GnomeCanvasItem    *item,
					    GdkEvent           *event,
					    gpointer            data);


static guint signals[SIGNAL_LAST] = { 0 };

static GnomeCanvasGroupClass *parent_class;


GType
gnome_month_item_get_type (void)
{
	static GType month_item_type = 0;

	if (!month_item_type) {
		static const GTypeInfo month_item_info = {
			sizeof (GnomeMonthItemClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) gnome_month_item_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (GnomeMonthItem),
			0,   /* n_preallocs */
			(GInstanceInitFunc) gnome_month_item_init,
			NULL
		};

		month_item_type = g_type_register_static (GNOME_TYPE_CANVAS_GROUP,
							  "GnomeMonthItem",
							  &month_item_info,
							  0);
	}

	return month_item_type;
}

static void
gnome_month_item_class_init (GnomeMonthItemClass *klass)
{
	GObjectClass         *object_class;
	GnomeCanvasItemClass *item_class;
	GParamSpec           *pspec;

	object_class = G_OBJECT_CLASS          (klass);
	item_class   = GNOME_CANVAS_ITEM_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->finalize     = gnome_month_item_finalize;
	object_class->dispose      = gnome_month_item_dispose;
	object_class->set_property = gnome_month_item_set_property;
	object_class->get_property = gnome_month_item_get_property;

	pspec = g_param_spec_uint ("year", NULL, NULL, 1, 2999, 1970, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_YEAR, pspec);
	pspec = g_param_spec_uint ("month", NULL, NULL, 0, 11, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_MONTH, pspec);
	pspec = g_param_spec_uint ("day", NULL, NULL, 1, 31, 1, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DAY, pspec);
	pspec = g_param_spec_double ("x", NULL, NULL, -1E+307, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_X, pspec);
	pspec = g_param_spec_double ("y", NULL, NULL, -1E+307, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_Y, pspec);
	pspec = g_param_spec_double ("width", NULL, NULL, 0, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_WIDTH, pspec);
	pspec = g_param_spec_double ("height", NULL, NULL, 0, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_HEIGHT, pspec);
	pspec = g_param_spec_enum ("anchor", NULL, NULL, GTK_TYPE_ANCHOR_TYPE, GTK_ANCHOR_WEST, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_ANCHOR, pspec);
	pspec = g_param_spec_double ("heading_padding", NULL, NULL, 0, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_HEAD_PADDING, pspec);
	pspec = g_param_spec_double ("day_padding", NULL, NULL, 0, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DAY_PADDING, pspec);
	pspec = g_param_spec_pointer ("day_names", NULL, NULL, G_PARAM_WRITABLE);
	g_object_class_install_property (object_class, ARG_DAY_NAMES, pspec);
	pspec = g_param_spec_double ("heading_height", NULL, NULL, 0, 1E+307, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_HEADING_HEIGHT, pspec);
	pspec = g_param_spec_enum ("heading_anchor", NULL, NULL, GTK_TYPE_ANCHOR_TYPE, GTK_ANCHOR_WEST, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_HEADING_ANCHOR, pspec);
	pspec = g_param_spec_enum ("day_anchor", NULL, NULL, GTK_TYPE_ANCHOR_TYPE, GTK_ANCHOR_WEST, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DAY_ANCHOR, pspec);
	pspec = g_param_spec_string ("heading_font", NULL, NULL, "Sans", G_PARAM_WRITABLE);
	g_object_class_install_property (object_class, ARG_HEAD_FONT, pspec);
	pspec = g_param_spec_pointer ("heading_font_desc", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_HEAD_FONT_DESC, pspec);
	pspec = g_param_spec_string ("day_font", NULL, NULL, "Sans", G_PARAM_WRITABLE);
	g_object_class_install_property (object_class, ARG_DAY_FONT, pspec);
	pspec = g_param_spec_pointer ("day_font_desc", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DAY_FONT_DESC, pspec);

	/* Signals */

	signals[SIGNAL_REORDERED] =
		g_signal_new ("reordered",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (GnomeMonthItemClass, reordered),
			      NULL, NULL,
			      gnomecal_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
}

static void
free_day_names (GnomeMonthItem *mitem)
{
	int i;

	for (i = 0; i < 7; i++) {
	    g_free (mitem->priv->day_names[i]);
	    mitem->priv->day_names[i] = NULL;
	}
}

static void
gnome_month_item_finalize (GObject *object)
{
	GnomeMonthItem *mitem;
	GnomeMonthItemPriv *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (object));

	mitem = GNOME_MONTH_ITEM (object);
	priv = mitem->priv;

	free_day_names (mitem);

	g_free (priv->head_color);
	g_free (priv->outline_color);
	g_free (priv->day_box_color);
	g_free (priv->day_color);

	g_free (priv);
	mitem->priv = NULL;

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
gnome_month_item_dispose (GObject *object)
{
	GnomeMonthItemPriv *priv;
	GConfClient *gconf;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (object));

	priv = GNOME_MONTH_ITEM (object)->priv;

	gconf = gconf_client_get_default ();

	if (priv->notify1) {
		gconf_client_notify_remove (gconf, priv->notify1);
		priv->notify1 = 0;
	}
	if (priv->notify2) {
		gconf_client_notify_remove (gconf, priv->notify2);
		priv->notify2 = 0;
	}

	g_object_unref (G_OBJECT (gconf));

	G_OBJECT_CLASS (parent_class)->dispose (object);
}

/* Calculates the minimum heading height based on the heading font size and
 * padding. It also calculates the minimum width of the month item based on
 * the width of the headings.
 */
static void
check_heading_sizes (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	GnomeCanvas *canvas;
	double m_height;
	double m_width;
	int width;
	int max_width;
	int i;
	gint fontsize;

	priv = mitem->priv;
	canvas = GNOME_CANVAS_ITEM (mitem)->canvas;

	/* Calculate minimum height */

	fontsize = pango_font_description_get_size (priv->head_font) / PANGO_SCALE + 1;
	m_height = fontsize + 2 * priv->head_padding;

	if (priv->head_height < m_height)
		priv->head_height = m_height;
	
	/* Go through each heading and remember the widest one */

	max_width = 0;

	for (i = 0; i < 7; i++) {
		PangoLayout *layout;

		layout = gtk_widget_create_pango_layout (GTK_WIDGET (canvas),
							 priv->day_names[i]);
		pango_layout_set_font_description (layout, priv->head_font);

		pango_layout_get_pixel_size (layout, &width, NULL);
		if (max_width < width)
			max_width = width;
	}

	m_width = 7 * (max_width + 2 * priv->head_padding);

	if (priv->width < m_width)
		priv->width = m_width;
}

/* Calculates the minimum width and height of the month item based on the day font size and padding.
 * Assumes that the minimum heading height has already been computed.
 */
static void
check_day_sizes (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	GnomeCanvas *canvas;
	PangoLayout *layout;
	double m_height;
	double m_width;
	int max_width, max_height;
	int i;

	canvas = GNOME_CANVAS_ITEM (mitem)->canvas;
	priv = mitem->priv;

	layout = gtk_widget_create_pango_layout (GTK_WIDGET (canvas), NULL);
	pango_layout_set_font_description (layout, priv->day_font);

	max_width  = 0;
	max_height = 0;

	for (i = 1; i < 32; i++) {
		char buf[10];
		int width, height;

		sprintf (buf, "%d", i);
		pango_layout_set_text (layout, buf, -1);
		pango_layout_get_pixel_size (layout, &width, &height);

		if (max_width < width)
			max_width = width;
		if (max_height < height)
			max_height = height;
	}

	m_width = 7 * (max_width + 2 * priv->day_padding);
	m_height = priv->head_height + 6 * (max_height + 2 * priv->day_padding);

	if (priv->width < m_width)
		priv->width = m_width;

	if (priv->height < m_height)
		priv->height = m_height;
}

/* Calculates the minimum size of the month item based on the font sizes and paddings.  If the
 * current size of the month item is smaller than the required minimum size, this function will
 * change the size to the appropriate values.
 */
static void
check_sizes (GnomeMonthItem *mitem)
{
	check_heading_sizes (mitem);
	check_day_sizes (mitem);
}

/* Recalculates the position of the toplevel calendar group based on the
 * logical position and anchor
 */
static void
reanchor (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	double x, y;

	priv = mitem->priv;

	x = priv->x;
	y = priv->y;

	switch (priv->anchor) {
	case GTK_ANCHOR_NW:
	case GTK_ANCHOR_W:
	case GTK_ANCHOR_SW:
		break;

	case GTK_ANCHOR_N:
	case GTK_ANCHOR_CENTER:
	case GTK_ANCHOR_S:
		x -= priv->width / 2;
		break;

	case GTK_ANCHOR_NE:
	case GTK_ANCHOR_E:
	case GTK_ANCHOR_SE:
		x -= priv->width;
		break;
	}

	switch (priv->anchor) {
	case GTK_ANCHOR_NW:
	case GTK_ANCHOR_N:
	case GTK_ANCHOR_NE:
		break;

	case GTK_ANCHOR_W:
	case GTK_ANCHOR_CENTER:
	case GTK_ANCHOR_E:
		y -= priv->height / 2;
		break;

	case GTK_ANCHOR_SW:
	case GTK_ANCHOR_S:
	case GTK_ANCHOR_SE:
		y -= priv->height;
		break;
	}

	/* Explicitly use the canvas group class prefix since the month
	 * item class has x and y arguments as well.
	 */

	gnome_canvas_item_set (GNOME_CANVAS_ITEM (mitem),
			       "GnomeCanvasGroup::x", x,
			       "GnomeCanvasGroup::y", y,
			       NULL);
}

/* Takes an anchor specification and the corners of a rectangle, and returns an anchored point with
 * respect to that rectangle.
 */
static void
get_label_anchor (GtkAnchorType anchor, double x1, double y1, double x2, double y2, double *x, double *y)
{
	switch (anchor) {
	case GTK_ANCHOR_NW:
	case GTK_ANCHOR_W:
	case GTK_ANCHOR_SW:
		*x = x1;
		break;

	case GTK_ANCHOR_N:
	case GTK_ANCHOR_CENTER:
	case GTK_ANCHOR_S:
		*x = (x1 + x2) / 2.0;
		break;

	case GTK_ANCHOR_NE:
	case GTK_ANCHOR_E:
	case GTK_ANCHOR_SE:
		*x = x2;
		break;
	}

	switch (anchor) {
	case GTK_ANCHOR_NW:
	case GTK_ANCHOR_N:
	case GTK_ANCHOR_NE:
		*y = y1;
		break;

	case GTK_ANCHOR_W:
	case GTK_ANCHOR_CENTER:
	case GTK_ANCHOR_E:
		*y = (y1 + y2) / 2.0;
		break;

	case GTK_ANCHOR_SW:
	case GTK_ANCHOR_S:
	case GTK_ANCHOR_SE:
		*y = y2;
		break;
	}
}

/* Resets the position of the day name headings in the calendar */
static void
reshape_headings (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	double width;
	int i;
	double x, y;

	priv = mitem->priv;

	width = priv->width / 7;

	for (i = 0; i < 7; i++) {
		/* Group */
		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_HEAD_GROUP + i],
				       "x", width * i,
				       "y", 0.0,
				       NULL);

		/* Box */
		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_HEAD_BOX + i],
				       "x1", 0.0,
				       "y1", 0.0,
				       "x2", width,
				       "y2", priv->head_height,
				       NULL);

		/* Label */
		get_label_anchor (priv->head_anchor,
				  priv->head_padding,
				  priv->head_padding,
				  width - priv->head_padding,
				  priv->head_height - priv->head_padding,
				  &x, &y);

		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_HEAD_LABEL + i],
				       "x", x,
				       "y", y,
				       "anchor", priv->head_anchor,
				       NULL);
	}
}

/* Resets the position of the days in the calendar */
static void
reshape_days (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	double width, height;
	double x, y;
	int row, col;
	int i;

	priv = mitem->priv;

	width = priv->width / 7;
	height = (priv->height - priv->head_height) / 6;

	i = 0;

	for (row = 0; row < 6; row++)
		for (col = 0; col < 7; col++) {
			/* Group */
			gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_GROUP + i],
					       "x", width * col,
					       "y", priv->head_height + height * row,
					       NULL);

			/* Box */
			gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_BOX + i],
					       "x1", 0.0,
					       "y1", 0.0,
					       "x2", width,
					       "y2", height,
					       NULL);

			/* Label */
			get_label_anchor (priv->day_anchor,
					  priv->day_padding,
					  priv->day_padding,
					  width - priv->day_padding,
					  height - priv->day_padding,
					  &x, &y);

			gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i],
					       "x", x,
					       "y", y,
					       "anchor", priv->day_anchor,
					       NULL);

			i++;
		}
}

/* Changes the positions and resizes the items in the calendar to match the new size of the
 * calendar.
 */
static void
reshape (GnomeMonthItem *mitem)
{
	check_sizes (mitem);
	reanchor (mitem);
	reshape_headings (mitem);
	reshape_days (mitem);
}

/* Sets the font for all the day headings */
static void
set_head_font (GnomeMonthItem *mitem)
{
	int i;

	for (i = 0; i < 7; i++)
		gnome_canvas_item_set (mitem->priv->items[GNOME_MONTH_ITEM_HEAD_LABEL + i],
				       "font_desc", mitem->priv->head_font,
				       NULL);
}

/* Sets the color for all the headings */
static void
set_head_color (GnomeMonthItem *mitem)
{
	int i;
	const gchar *outline_color, *head_color;

	outline_color = mitem->priv->outline_color;
	head_color    = mitem->priv->head_color;

	if (!pango_color_parse (NULL, outline_color))
		outline_color = "#000";
	if (!pango_color_parse (NULL, head_color))
		head_color = "#000";

	for (i = 0; i < 7; i++) {
		gnome_canvas_item_set (mitem->priv->items[GNOME_MONTH_ITEM_HEAD_BOX + i],
				       "fill_color", outline_color,
				       NULL);

		gnome_canvas_item_set (mitem->priv->items[GNOME_MONTH_ITEM_HEAD_LABEL + i],
				       "fill_color", head_color,
				       NULL);
	}
}

/* Creates the items for the day name headings */
static void
create_headings (GnomeCanvasGroup *parent)
{
	int i;
	GnomeMonthItem *mitem = GNOME_MONTH_ITEM (parent);

	/* Just create the items; they will be positioned and configured by
	 * a call to reshape().
	 */

	for (i = 0; i < 7; i++) {
		GnomeCanvasGroup *group;

		/* Group */
		mitem->priv->items[GNOME_MONTH_ITEM_HEAD_GROUP + i] =
			gnome_canvas_item_new (parent,
					       GNOME_TYPE_CANVAS_GROUP,
					       NULL);
		group = GNOME_CANVAS_GROUP (mitem->priv->items[GNOME_MONTH_ITEM_HEAD_GROUP + i]);

		/* Box */
		mitem->priv->items[GNOME_MONTH_ITEM_HEAD_BOX + i] =
			gnome_canvas_item_new (group,
					       GNOME_TYPE_CANVAS_RECT,
					       NULL);

		/* Label */
		mitem->priv->items[GNOME_MONTH_ITEM_HEAD_LABEL + i] =
			gnome_canvas_item_new (group,
					       GNOME_TYPE_CANVAS_TEXT,
					       "text", "",
					       NULL);
	}

	set_head_font  (mitem);
	set_head_color (mitem);
}

/* Returns the number of leap years since year 1 up to (but not including) the specified year */
static int
leap_years_up_to (int year)
{
	return (year / 4					/* trivial leapness */
		- ((year > 1700) ? (year / 100 - 17) : 0)	/* minus centuries since 1700 */
		+ ((year > 1600) ? ((year - 1600) / 400) : 0));	/* plus centuries since 1700 divisible by 400 */
}

/* Returns whether the specified year is a leap year */
static int
is_leap_year (int year)
{
	if (year <= 1752)
		return !(year % 4);
	else
		return (!(year % 4) && (year % 100)) || !(year % 400);
}

/* Returns the 1-based day number within the year of the specified date */
static int
day_in_year (int day, int month, int year)
{
	int is_leap, i;

	is_leap = is_leap_year (year);

	for (i = 0; i < month; i++)
		day += days_in_month [is_leap][i];

	return day;
}

/* Returns the day of the week (zero-based, zero is Sunday) for the specified date.  For the days
 * that were removed on the Gregorian reformation, it returns Thursday.
 */
static int
day_in_week (int day, int month, int year)
{
	int n;

	n = (year - 1) * 365 + leap_years_up_to (year - 1) + day_in_year (day, month, year);

	if (n < REFORMATION_DAY)
		return (n - 1 + SATURDAY) % 7;

	if (n >= (REFORMATION_DAY + MISSING_DAYS))
		return (n - 1 + SATURDAY - MISSING_DAYS) % 7;

	return THURSDAY;
}

/* Fills the 42-element days array with the day numbers for the specified month.  Slots outside the
 * bounds of the month are filled with zeros.  The starting and ending indexes of the days are
 * returned in the start and end arguments.
 */
static void
build_month (int month, int year, int start_on_monday, int *days, int *start, int *end)
{
	int i;
	int d_month, d_week;

	/* Note that months are zero-based, so September is month 8 */

	if ((year == 1752) && (month == 8)) {
		memcpy (days, sept_1752, NUMBER_OF_DAYS * sizeof (int));

		if (start)
			*start = SEPT_1752_START;

		if (end)
			*end = SEPT_1752_END;

		return;
	}

	for (i = 0; i < NUMBER_OF_DAYS; i++)
		days[i] = 0;

	d_month = days_in_month[is_leap_year (year)][month];
	d_week = day_in_week (1, month, year);

	if (start_on_monday)
		d_week = (d_week + 6) % 7;

	for (i = 0; i < d_month; i++)
		days[d_week + i] = i + 1;

	if (start)
		*start = d_week;

	if (end)
		*end = d_week + d_month - 1;
}

/* Set the day numbers in the monthly calendar */
static void
set_days (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	int i;
	int start, end;
	char buf[100];

	priv = mitem->priv;

	build_month (priv->month, priv->year, priv->start_on_monday, priv->day_numbers, &start, &end);

	/* Clear days before start of month */

	for (i = 0; i < start; i++)
		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i],
				       "text", "",
				       NULL);

	/* Set days of month */

	for (; start <= end; start++, i++) {
		sprintf (buf, "%d", priv->day_numbers[start]);
		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i],
				       "text", buf,
				       NULL);
	}

	/* Clear days after end of month */

	for (; i < NUMBER_OF_DAYS; i++)
		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i],
				       "text", "",
				       NULL);
}

/* Sets the font for all the day numbers */
static void
set_day_font (GnomeMonthItem *mitem)
{
	int i;

	for (i = 0; i < NUMBER_OF_DAYS; i++)
		gnome_canvas_item_set (mitem->priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i],
				       "font_desc", mitem->priv->day_font,
				       NULL);
}

/* Sets the color for all the day items */
static void
set_day_color (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	const gchar *outline_color, *day_box_color, *day_color;
	int i;

	priv = mitem->priv;

	outline_color = priv->outline_color;
	day_box_color = priv->day_box_color;
	day_color     = priv->day_color;

	if (!pango_color_parse (NULL, outline_color))
		outline_color = "#000";
	if (!pango_color_parse (NULL, day_box_color))
		day_box_color = "#000";
	if (!pango_color_parse (NULL, day_color))
		day_color = "#000";

	for (i = 0; i < NUMBER_OF_DAYS; i++) {
		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_BOX + i],
				       "outline_color", outline_color,
				       "fill_color",    day_box_color,
				       NULL);

		gnome_canvas_item_set (priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i],
				       "fill_color", day_color,
				       NULL);
	}
}

/* Creates the items for the days */
static void
create_days (GnomeCanvasGroup *parent)
{
	int i;
	GnomeMonthItem *mitem = GNOME_MONTH_ITEM (parent);

	/* Just create the items; they will be positioned and configured by a call to reshape() */

	for (i = 0; i < NUMBER_OF_DAYS; i++) {
		GnomeCanvasGroup *group;

		/* Group */
		mitem->priv->items[GNOME_MONTH_ITEM_DAY_GROUP + i] =
			gnome_canvas_item_new (parent,
					       GNOME_TYPE_CANVAS_GROUP,
					       NULL);
		group = GNOME_CANVAS_GROUP (mitem->priv->items[GNOME_MONTH_ITEM_DAY_GROUP + i]);

		/* Box */
		mitem->priv->items[GNOME_MONTH_ITEM_DAY_BOX + i] =
			gnome_canvas_item_new (group,
					       GNOME_TYPE_CANVAS_RECT,
					       NULL);

		/* Label */
		mitem->priv->items[GNOME_MONTH_ITEM_DAY_LABEL + i] =
			gnome_canvas_item_new (group,
					       GNOME_TYPE_CANVAS_TEXT,
					       NULL);
	}

	set_day_font (mitem);
	set_day_color (mitem);
	set_days (mitem);
}

/* Resets the text of the day name headings */
static void
set_day_names (GnomeMonthItem *mitem)
{
	int i;

	for (i = 0; i < 7; i++)
		gnome_canvas_item_set (mitem->priv->items[GNOME_MONTH_ITEM_HEAD_LABEL + i],
				       "text", mitem->priv->day_names[mitem->priv->start_on_monday ? ((i + 1) % 7) : i],
				       NULL);
}

/* Creates all the canvas items that make up the calendar */
static void
create_items (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	int i;

	priv = mitem->priv;

	priv->items = g_new (GnomeCanvasItem *, GNOME_MONTH_ITEM_LAST);

	create_headings (GNOME_CANVAS_GROUP (mitem));
	create_days (GNOME_CANVAS_GROUP (mitem));

	/* Initialize by default to short day names */

	for (i = 0; i < 7; i++) {
		gchar buffer[40];
		struct tm tm;

		memset (&tm, 0, sizeof (tm));
		tm.tm_wday = i;

		strftime (buffer, sizeof (buffer), "%a", &tm);
		priv->day_names[i] = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
	}
	
	set_day_names (mitem);
	reshape (mitem);
}

static void
gnome_month_item_config_changed (GConfClient	*gconf,
				 guint		 cnxn_id,
				 GConfEntry	*entry,
				 gpointer	 data)
{
	GnomeMonthItem *mitem;
	const gchar *key;

	g_return_if_fail (entry != NULL);
	g_return_if_fail (data != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (data));

	mitem = GNOME_MONTH_ITEM (data);

	key = gconf_entry_get_key (entry);

	if (strcmp (key, CONFIG_GENERAL_WEEK_START) == 0) {
		mitem->priv->start_on_monday =
			!gconf_value_get_bool (gconf_entry_get_value (entry));

		set_day_names (mitem);
		set_days (mitem);

		g_signal_emit (G_OBJECT (mitem), signals[SIGNAL_REORDERED], 0);
	}
}

static void
gnome_month_item_color_changed  (GConfClient	*gconf,
				 guint		 cnxn_id,
				 GConfEntry	*entry,
				 gpointer	 data)
{
	GnomeMonthItem     *mitem;
	GnomeMonthItemPriv *priv;
	const gchar *key;
	const gchar *value;

	g_return_if_fail (entry != NULL);
	g_return_if_fail (data != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (data));

	mitem = GNOME_MONTH_ITEM (data);
	priv = mitem->priv;

	key   = gconf_entry_get_key (entry);
	value = gconf_value_get_string (gconf_entry_get_value (entry));

	if (strcmp (key, CONFIG_COLOR_HEADINGS) == 0) {
		g_free (priv->head_color);
		priv->head_color = g_strdup (value);
		set_head_color (mitem);
	} else if (strcmp (key, CONFIG_COLOR_OUTLINE) == 0) {
		g_free (priv->outline_color);
		priv->outline_color = g_strdup (value);
		set_head_color (mitem);
		set_day_color (mitem);
	} else if (strcmp (key, CONFIG_COLOR_EMPTY_DAYS) == 0) {
		g_free (priv->day_box_color);
		priv->day_box_color = g_strdup (value);
		set_day_color (mitem);
	} else if (strcmp (key, CONFIG_COLOR_DAY_NUMBERS) == 0) {
		g_free (priv->day_color);
		priv->day_color = g_strdup (value);
		set_day_color (mitem);
	}

	g_signal_emit (G_OBJECT (mitem), signals[SIGNAL_REORDERED], 0);
}

static void
gnome_month_item_init (GnomeMonthItem *mitem)
{
	GnomeMonthItemPriv *priv;
	time_t t;
	struct tm tm;
	GConfClient *gconf;

	g_return_if_fail (mitem != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (mitem));

	priv = g_new0 (GnomeMonthItemPriv, 1);
	mitem->priv = priv;

	/* Initialize to the current month by default */

	t = time (NULL);
	tm = *localtime (&t);

	priv->year = tm.tm_year + 1900;
	priv->month = tm.tm_mon;

	priv->x = 0.0;
	priv->y = 0.0;
	priv->width = 150.0; /* not unreasonable defaults, I hope */
	priv->height = 100.0;
	priv->anchor = GTK_ANCHOR_NW;
	priv->head_padding = 0.0;
	priv->day_padding = 2.0;
	priv->head_height = 14.0;
	priv->head_anchor = GTK_ANCHOR_CENTER;
	priv->day_anchor = GTK_ANCHOR_CENTER;

	/* Load the default fonts */

	priv->head_font = pango_font_description_from_string (DEFAULT_FONT);
	g_assert (priv->head_font != NULL);
	priv->day_font  = pango_font_description_from_string (DEFAULT_FONT);
	g_assert (priv->day_font != NULL);

	/* Load preferences */

	gconf = gconf_client_get_default ();

	gconf_client_add_dir (gconf, CONFIG_GENERAL_DIR,
			      GCONF_CLIENT_PRELOAD_NONE, NULL);
	gconf_client_add_dir (gconf, CONFIG_COLOR_DIR,
			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

	priv->start_on_monday =
		!gconf_client_get_bool (gconf, CONFIG_GENERAL_WEEK_START, NULL);
	priv->head_color =
		gncal_config_get_color (gconf, CONFIG_COLOR_HEADINGS,    DEFAULT_COLOR_HEADINGS);
	priv->outline_color =
		gncal_config_get_color (gconf, CONFIG_COLOR_OUTLINE,     DEFAULT_COLOR_OUTLINE);
	priv->day_box_color =
		gncal_config_get_color (gconf, CONFIG_COLOR_EMPTY_DAYS,  DEFAULT_COLOR_EMPTY_DAYS);
	priv->day_color =
		gncal_config_get_color (gconf, CONFIG_COLOR_DAY_NUMBERS, DEFAULT_COLOR_DAY_NUMBERS);

	/* Preferences notifications */

	priv->notify1 =
		gconf_client_notify_add (gconf, CONFIG_GENERAL_WEEK_START,
					 gnome_month_item_config_changed, mitem,
					 NULL, NULL);
	priv->notify2 =
		gconf_client_notify_add (gconf, CONFIG_COLOR_DIR,
					 gnome_month_item_color_changed, mitem,
					 NULL, NULL);

	g_object_unref (G_OBJECT (gconf));
}

GnomeCanvasItem *
gnome_month_item_new (GnomeCanvasGroup *parent)
{
	GnomeMonthItem *mitem;

	g_return_val_if_fail (parent != NULL, NULL);
	g_return_val_if_fail (GNOME_IS_CANVAS_GROUP (parent), NULL);

	mitem = GNOME_MONTH_ITEM (gnome_canvas_item_new (parent,
							 gnome_month_item_get_type (),
							 NULL));

	gnome_month_item_construct (mitem);

	return GNOME_CANVAS_ITEM (mitem);
}

void
gnome_month_item_construct (GnomeMonthItem *mitem)
{
	gint i;

	g_return_if_fail (mitem != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (mitem));

	create_items (mitem);

	/* Connect the appropriate signals to perform prelighting */

	for (i = 0; i < 42; i++) {
		GnomeCanvasItem *day_group;

		day_group = gnome_month_item_num2child (GNOME_MONTH_ITEM (mitem), GNOME_MONTH_ITEM_DAY_GROUP + i);
		g_signal_connect (G_OBJECT (day_group), "event",
				  G_CALLBACK (gnome_month_item_day_event), mitem);
	}
}

static void
gnome_month_item_set_property (GObject      *object,
			       guint         property_id,
			       const GValue *value,
			       GParamSpec   *pspec)
{
	GnomeMonthItem *mitem;
	GnomeMonthItemPriv *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GNOME_IS_MONTH_ITEM (object));

	mitem = GNOME_MONTH_ITEM (object);
	priv = mitem->priv;

	switch (property_id) {
	case ARG_YEAR:
		priv->year = g_value_get_uint (value);
		set_days (mitem);
		break;

	case ARG_MONTH:
		priv->month = g_value_get_uint (value);
		set_days (mitem);
		break;

	case ARG_DAY:
		priv->day = g_value_get_uint (value);
		break;

	case ARG_X:
		priv->x = g_value_get_double (value);
		reanchor (mitem);
		break;

	case ARG_Y:
		priv->y = g_value_get_double (value);
		reanchor (mitem);
		break;

	case ARG_WIDTH:
		priv->width = g_value_get_double (value);
		reshape (mitem);
		break;

	case ARG_HEIGHT:
		priv->height = g_value_get_double (value);
		reshape (mitem);
		break;

	case ARG_ANCHOR:
		priv->anchor = g_value_get_enum (value);
		reanchor (mitem);
		break;

	case ARG_HEAD_PADDING:
		priv->head_padding = g_value_get_double (value);
		reshape (mitem);
		break;

	case ARG_DAY_PADDING:
		priv->day_padding = g_value_get_double (value);
		reshape (mitem);
		break;

	case ARG_DAY_NAMES: {
		gchar **day_names;
		int i;

		day_names = g_value_get_pointer (value);

		/* First, check that none of the names is null */

		for (i = 0; i < 7; i++) {
			g_return_if_fail (day_names[i] != NULL);
		}

		/* Set the new names */

		free_day_names (mitem);
		for (i = 0; i < 7; i++)
			priv->day_names[i] = g_strdup (day_names[i]);
		set_day_names (mitem);

		reshape (mitem);

		break;
	}

	case ARG_HEADING_HEIGHT:
		priv->head_height = g_value_get_double (value);
		reshape (mitem);
		break;

	case ARG_HEADING_ANCHOR:
		priv->head_anchor = g_value_get_enum (value);
		reshape (mitem);
		break;

	case ARG_DAY_ANCHOR:
		priv->day_anchor = g_value_get_enum (value);
		reshape (mitem);
		break;

	case ARG_HEAD_FONT:
		pango_font_description_free (priv->head_font);
		priv->head_font = pango_font_description_from_string (g_value_get_string (value));

		g_return_if_fail (priv->head_font != NULL);

		set_head_font (mitem);
		reshape (mitem);

		break;

	case ARG_HEAD_FONT_DESC:
		pango_font_description_free (priv->head_font);
		priv->head_font = pango_font_description_copy (g_value_get_pointer (value));
		break;

	case ARG_DAY_FONT:
		pango_font_description_free (priv->day_font);
		priv->day_font = pango_font_description_from_string (g_value_get_string (value));

		g_return_if_fail (priv->day_font != NULL);

		set_day_font (mitem);
		reshape (mitem);

		break;

	case ARG_DAY_FONT_DESC:
		pango_font_description_free (priv->day_font);
		priv->day_font = pango_font_description_copy (g_value_get_pointer (value));
		break;

	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
gnome_month_item_get_property (GObject    *object,
			       guint       property_id,
			       GValue     *value,
			       GParamSpec *pspec)
{
	GnomeMonthItemPriv *priv;

	priv = GNOME_MONTH_ITEM (object)->priv;

	switch (property_id) {
	case ARG_YEAR:
		g_value_set_uint (value, priv->year);
		break;

	case ARG_MONTH:
		g_value_set_uint (value, priv->month);
		break;

	case ARG_DAY:
		g_value_set_uint (value, priv->day);
		break;

	case ARG_X:
		g_value_set_double (value, priv->x);
		break;

	case ARG_Y:
		g_value_set_double (value, priv->y);
		break;

	case ARG_WIDTH:
		g_value_set_double (value, priv->width);
		break;

	case ARG_HEIGHT:
		g_value_set_double (value, priv->height);
		break;

	case ARG_ANCHOR:
		g_value_set_enum (value, priv->anchor);
		break;

	case ARG_HEAD_PADDING:
		g_value_set_double (value, priv->head_padding);
		break;

	case ARG_DAY_PADDING:
		g_value_set_double (value, priv->day_padding);
		break;

	case ARG_HEADING_HEIGHT:
		g_value_set_double (value, priv->head_height);
		break;

	case ARG_HEADING_ANCHOR:
		g_value_set_enum (value, priv->head_anchor);
		break;

	case ARG_DAY_ANCHOR:
		g_value_set_enum (value, priv->day_anchor);
		break;

	case ARG_HEAD_FONT_DESC:
		g_value_set_pointer (value, priv->head_font);
		break;

	case ARG_DAY_FONT_DESC:
		g_value_set_pointer (value, priv->day_font);
		break;

	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

GnomeCanvasItem *
gnome_month_item_num2child (GnomeMonthItem *mitem, gsize child_num)
{
	g_return_val_if_fail (mitem != NULL, NULL);
	g_return_val_if_fail (GNOME_IS_MONTH_ITEM (mitem), NULL);
	g_return_val_if_fail (child_num < GNOME_MONTH_ITEM_LAST, NULL);

	return mitem->priv->items[child_num];
}

gint
gnome_month_item_child2num (GnomeMonthItem *mitem, GnomeCanvasItem *child)
{
	gint i;

	g_return_val_if_fail (mitem != NULL, -1);
	g_return_val_if_fail (GNOME_IS_MONTH_ITEM (mitem), -1);
	g_return_val_if_fail (child != NULL, -1);
	g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (child), -1);

	for (i = 0; i < GNOME_MONTH_ITEM_LAST; i++)
		if (mitem->priv->items[i] == child)
			return i;

	return -1;
}

gint
gnome_month_item_num2day (GnomeMonthItem *mitem, gsize child_num)
{
	g_return_val_if_fail (mitem != NULL, 0);
	g_return_val_if_fail (GNOME_IS_MONTH_ITEM (mitem), 0);

	if ((child_num >= GNOME_MONTH_ITEM_DAY_GROUP) &&
	    (child_num < GNOME_MONTH_ITEM_LAST)) {
		child_num = (child_num - GNOME_MONTH_ITEM_DAY_GROUP) % NUMBER_OF_DAYS;
		return mitem->priv->day_numbers[child_num];
	} else
		return 0;
}

gint
gnome_month_item_day2index (GnomeMonthItem *mitem, guint day_num)
{
	guint i;

	g_return_val_if_fail (mitem != NULL, -1);
	g_return_val_if_fail (GNOME_IS_MONTH_ITEM (mitem), -1);
	g_return_val_if_fail (day_num >= 1, -1);

	/* Find first day of month */

	for (i = 0; mitem->priv->day_numbers[i] == 0; i++)
		;

	/* Find the specified day */

	for (; (mitem->priv->day_numbers[i] != 0) && (i < NUMBER_OF_DAYS); i++)
		if (mitem->priv->day_numbers[i] == day_num)
			return i;

	/* Bail out */

	return -1;
}

/* Handles EnterNotify and LeaveNotify events from the month item's day groups, and performs
 * appropriate prelighting.
 */
static gboolean
gnome_month_item_day_event (GnomeCanvasItem *item, GdkEvent *event, gpointer data)
{
	GnomeMonthItem *mitem;
	GnomeCanvasItem *box;
	int child_num, day;
	GConfClient *gconf;
	gchar *color = NULL;

	/* We only accept enters and leaves */

	if (!((event->type == GDK_ENTER_NOTIFY) || (event->type == GDK_LEAVE_NOTIFY)))
		return FALSE;

	/* Get index information */

	mitem = GNOME_MONTH_ITEM (data);
	child_num = gnome_month_item_child2num (mitem, item);
	day = gnome_month_item_num2day (mitem, child_num);

	if (day == 0)
		return FALSE; /* it was a day outside the month's range */

	child_num -= GNOME_MONTH_ITEM_DAY_GROUP;
	box = gnome_month_item_num2child (mitem, GNOME_MONTH_ITEM_DAY_BOX + child_num);

	/* Get the color */

	gconf = gconf_client_get_default ();

	/* Now actually set the proper color in the item */

	switch (event->type) {
	case GDK_ENTER_NOTIFY:
		color = gncal_config_get_color (gconf, CONFIG_COLOR_DAY_HIGHLIGHT, DEFAULT_COLOR_DAY_HIGHLIGHT);
		break;

	case GDK_LEAVE_NOTIFY: {
		gint type;

		type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (box), "bg_type"));
		switch (type) {
		case 0:
			color = gncal_config_get_color (gconf, CONFIG_COLOR_EMPTY_DAYS, DEFAULT_COLOR_EMPTY_DAYS);
			break;
		case 1:
			color = gncal_config_get_color (gconf, CONFIG_COLOR_APPOINTMENTS, DEFAULT_COLOR_APPOINTMENTS);
			break;
		default:
			g_object_unref (G_OBJECT (gconf));
			g_assert_not_reached ();
		}

		break;
	}

	default:
		g_object_unref (G_OBJECT (gconf));
		g_return_val_if_reached (FALSE);
	}

	gnome_canvas_item_set (box, "fill_color", color, NULL);

	g_free (color);
	g_object_unref (G_OBJECT (gconf));

	return TRUE;
}
