/* Generic ICalendar Object
 * Copyright (C) 2002  Sebastian Rittau <srittau@jroger.in-berlin.de>
 *
 * $Id: ical-component.c,v 1.1.2.2 2002/04/27 14:29:07 srittau Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include <glib.h>

#include "gnomecal-marshal.h"
#include "ical-component.h"


enum {
	CHANGED_SIGNAL,
	LAST_SIGNAL
};

enum {
	ARG_CATEGORIES = 1,
	ARG_CLASS,
	ARG_DESCRIPTION,
	ARG_PRIORITY,
	ARG_STATUS,
	ARG_SUMMARY,

	ARG_DTEND,
	ARG_DUE,
	ARG_DTSTART,
	ARG_TRANSP,

	ARG_ORGANIZER,
	ARG_UID,

	ARG_CREATED,
	ARG_DTSTAMP,
	ARG_LAST_MODIFIED,
	ARG_SEQUENCE
};

struct _ICalComponentPriv
{
	/* Descriptive Properties (RFC 2445, chapter 4.8.1) */

	/* Attachment (4.8.1.1) is not supported, yet */
	GList		*categories;	/* Categories,		4.8.1.2 */
	gchar		*klass;		/* Classification,	4.8.1.3  */
	/* Comment (4.8.1.4) is not supported, yet */
	gchar		*description;	/* Description,		4.8.1.5  */
	/* Geographic Position (4.8.1.6) is not supported, yet */
	/* Location (4.8.1.7) is not supported, yet */
	/* Percent Complete (4.8.1.8) is not supported, yet */
	guint		 priority;	/* Priority,		4.8.1.9  */
	/* Resources (4.8.1.10) is not supported, yet */
	ICalStatus	 status;	/* Status,		4.8.1.11 */
	/* Status (4.8.1.11) is not supported, yet */
	gchar		*summary;	/* Summary,		4.8.1.12 */

	/* Date and Time Properties (RFC 2445, chapter 4.8.2) */

	/* Date/Time Completed (4.8.2.1) is not supported, yet */
	struct tm	*dtend;		/* Date/Time End,	4.8.2.2  */
	struct tm	*due;		/* Date/Time Due,	4.8.2.3  */
	struct tm	*dtstart;	/* Date/Time Start,	4.8.2.4  */
	/* Duration (4.8.2.5) is not supported, yet */
	/* Free/Busy Time (4.8.2.6) is not supported, yet */
	gboolean	 transp;	/* Time Transparency,	4.8.2.7  */

	/* Time Zone Properties (RFC 2445, chapter 4.8.3) */

	/* No properties supported, yet */

	/* Relationship Properties (RFC 2445, chapter 4.8.4) */

	/* Attendee (4.8.4.1) is not supported, yet */
	/* Contact (4.8.4.2) is not supported, yet */
	gchar		*organizer;	/* Organizer,		4.8.4.3  */
	/* Recurrence ID (4.8.4.4) is not supported, yet */
	/* Related To (4.8.4.5) is not supported, yet */
	/* Uniform Resource Location (4.8.4.6) is not supported, yet */
	gchar		*uid;		/* Unique Identifier,	4.8.4.7  */

	/* Recurrence Properties (RFC 2445, chapter 4.8.5) */

	/* No properties supported, yet */

	/* Alarm Properties (RFC 2445, chapter 4.8.6) */

	/* No properties supported, yet */

	/* Change Management Properties (RFC 2445, chapter 4.8.7) */

	struct tm	*created;	/* Date/Time Created,	4.8.7.1  */
	struct tm	*dtstamp;	/* Date/Time Stamp,	4.8.7.2  */
	struct tm	*last_modified;	/* Last Modified,	4.8.7.3  */
	gint32		 sequence;	/* Sequence,		4.8.7.4  */

	/* Miscellaneous Properties (RFC 2445, chapter 4.8.8) */

	/* No properties supported, yet */
};


static void ical_component_class_init	(ICalComponentClass *klass);
static void ical_component_init		(ICalComponent *item);
static void ical_component_finalize	(GObject	*object);
static void ical_component_set_property	(GObject	*object,
					 guint		 property_id,
					 const GValue	*value,
					 GParamSpec	*pspec);
static void ical_component_get_property	(GObject	*object,
					 guint		 property_id,
					 GValue		*value,
					 GParamSpec	*pspec);
static void ical_component_copy_categories (ICalComponent *item, GList *list);
static void ical_component_free_categories (ICalComponent *item);


static GObjectClass *parent_class;

static int item_signals[LAST_SIGNAL] = { 0 };

/*
 * Class and Object Handling
 */

GType
ical_component_get_type (void)
{
	static GType ical_component_type = 0;

	if (!ical_component_type) {
		static const GTypeInfo ical_component_info = {
			sizeof (ICalComponentClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) ical_component_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (ICalComponent),
			5,    /* n_preallocs */
			(GInstanceInitFunc) ical_component_init,
		};

		ical_component_type = g_type_register_static (G_TYPE_OBJECT,
							      "ICalComponent",
							      &ical_component_info,
							      0);
	}

	return ical_component_type;
}

static void
ical_component_class_init (ICalComponentClass *klass)
{
	GObjectClass *object_class;
	GParamSpec   *pspec;

	object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	item_signals[CHANGED_SIGNAL] =
		g_signal_new ("changed",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (ICalComponentClass, changed),
			      NULL, NULL,
			      gnomecal_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);

	object_class->finalize     = ical_component_finalize;
	object_class->set_property = ical_component_set_property;
	object_class->get_property = ical_component_get_property;

	pspec = g_param_spec_pointer ("categories", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_CATEGORIES, pspec);
	pspec = g_param_spec_string ("class", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_CLASS, pspec);
	pspec = g_param_spec_string ("description", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DESCRIPTION, pspec);
	pspec = g_param_spec_uint ("priority", NULL, NULL, 0, 9, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_PRIORITY, pspec);
	pspec = g_param_spec_int ("status", NULL, NULL, ICAL_EVENT_CANCELLED, ICAL_JOURNAL_FINAL, ICAL_EVENT_CANCELLED, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_STATUS, pspec);
	pspec = g_param_spec_string ("summary", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_SUMMARY, pspec);

	pspec = g_param_spec_pointer ("dtend", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DTEND, pspec);
	pspec = g_param_spec_pointer ("due", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DUE, pspec);
	pspec = g_param_spec_pointer ("dtstart", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DTSTART, pspec);
	pspec = g_param_spec_boolean ("transp", NULL, NULL, FALSE, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_TRANSP, pspec);

	pspec = g_param_spec_string ("organizer", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_ORGANIZER, pspec);
	pspec = g_param_spec_string ("uid", NULL, NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_UID, pspec);

	pspec = g_param_spec_pointer ("created", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_CREATED, pspec);
	pspec = g_param_spec_pointer ("dtstamp", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_DTSTAMP, pspec);
	pspec = g_param_spec_pointer ("last-modified", NULL, NULL, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_LAST_MODIFIED, pspec);
	pspec = g_param_spec_long ("sequence", NULL, NULL, 0, 0x7fffffff, 0, G_PARAM_READWRITE);
	g_object_class_install_property (object_class, ARG_SEQUENCE, pspec);
}

static void
ical_component_init (ICalComponent *item)
{
	g_return_if_fail (item != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (item));

	item->priv = g_new0 (ICalComponentPriv, 1);
}

static void
ical_component_finalize (GObject *object)
{
	ICalComponent *item;

	g_return_if_fail (object != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (object));

	item = ICAL_COMPONENT (object);

	if (item->priv) {
		ical_component_free_categories (item);

		g_free (item->priv->klass);
		g_free (item->priv->description);
		g_free (item->priv->summary);
		g_free (item->priv->organizer);
		g_free (item->priv->dtend);
		g_free (item->priv->due);
		g_free (item->priv->dtstart);
		g_free (item->priv->created);
		g_free (item->priv->dtstamp);
		g_free (item->priv->last_modified);
		g_free (item->priv);
		item->priv = NULL;
	}

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

/*
 * Overloaded Methods
 */

static void
ical_component_set_property_string (gchar **dst, const GValue *value)
{
	const gchar *string;

	string = g_value_get_string (value);
	g_free (*dst);
	*dst = g_strdup (string);
}

static void
ical_component_set_property_tm (struct tm **tm, const GValue *value)
{
	g_free (*tm);
	*tm = g_memdup (g_value_get_pointer (value), sizeof (struct tm));
}

static void
ical_component_set_property (GObject      *object,
			     guint         property_id,
			     const GValue *value,
			     GParamSpec   *pspec)
{
	ICalComponent     *item;
	ICalComponentPriv *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (object));

	item = ICAL_COMPONENT (object);
	priv = item->priv;

	switch (property_id) {
	case ARG_CATEGORIES:
		ical_component_copy_categories (item, g_value_get_pointer (value));
		break;
	case ARG_CLASS:
		ical_component_set_property_string (&priv->klass, value);
		break;
	case ARG_DESCRIPTION:
		ical_component_set_property_string (&priv->description, value);
		break;
	case ARG_PRIORITY:
		priv->priority = g_value_get_uint (value);
		break;
	case ARG_STATUS:
		priv->status = g_value_get_int (value);
		break;
	case ARG_SUMMARY:
		ical_component_set_property_string (&priv->summary, value);
		break;

	case ARG_DTEND:
		ical_component_set_property_tm (&priv->dtend, value);
		break;
	case ARG_DUE:
		ical_component_set_property_tm (&priv->due, value);
		break;
	case ARG_DTSTART:
		ical_component_set_property_tm (&priv->dtstart, value);
		break;
	case ARG_TRANSP:
		priv->transp = g_value_get_boolean (value);
		break;

	case ARG_ORGANIZER:
		ical_component_set_property_string (&priv->organizer, value);
		break;
	case ARG_UID:
		ical_component_set_property_string (&priv->uid, value);
		break;

	case ARG_CREATED:
		ical_component_set_property_tm (&priv->created, value);
		break;
	case ARG_DTSTAMP:
		ical_component_set_property_tm (&priv->dtstamp, value);
		break;
	case ARG_LAST_MODIFIED:
		ical_component_set_property_tm (&priv->last_modified, value);
		break;
	case ARG_SEQUENCE:
		priv->sequence = g_value_get_long (value);
		break;

        default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		return;
	}

	g_signal_emit (object, item_signals[CHANGED_SIGNAL], 0);
}

static void
ical_component_get_property (GObject    *object,
			     guint       property_id,
			     GValue     *value,
			     GParamSpec *pspec)
{
	ICalComponent     *item;
	ICalComponentPriv *priv;

	g_return_if_fail (object != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (object));

	item = ICAL_COMPONENT (object);
	priv = item->priv;

	switch (property_id) {
	case ARG_CATEGORIES:
		g_value_set_pointer (value, item->priv->categories);
		break;
	case ARG_CLASS:
		g_value_set_string (value, priv->klass);
		break;
	case ARG_DESCRIPTION:
		g_value_set_string (value, priv->description);
		break;
	case ARG_PRIORITY:
		g_value_set_uint (value, priv->priority);
		break;
	case ARG_STATUS:
		g_value_set_int (value, priv->status);
		break;
	case ARG_SUMMARY:
		g_value_set_string (value, priv->summary);
		break;

	case ARG_DTEND:
		g_value_set_pointer (value, priv->dtend);
		break;
	case ARG_DUE:
		g_value_set_pointer (value, priv->due);
		break;
	case ARG_DTSTART:
		g_value_set_pointer (value, priv->dtstart);
		break;
	case ARG_TRANSP:
		g_value_set_boolean (value, priv->transp);
		break;

	case ARG_ORGANIZER:
		g_value_set_string (value, priv->organizer);
		break;
	case ARG_UID:
		g_value_set_string (value, priv->uid);
		break;

	case ARG_CREATED:
		g_value_set_pointer (value, priv->created);
		break;
	case ARG_DTSTAMP:
		g_value_set_pointer (value, priv->dtstamp);
		break;
	case ARG_LAST_MODIFIED:
		g_value_set_pointer (value, priv->last_modified);
		break;
	case ARG_SEQUENCE:
		g_value_set_long (value, priv->sequence);
		break;

        default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

/*
 * Utility Functions
 */

#define CHUNK_SIZE (1024)
static gchar *
ical_component_get_hostname (void)
{
	gchar *hostname = NULL;
	gsize  sz = 0;

	do {
		sz += CHUNK_SIZE;
		hostname = g_realloc (hostname, sz);
	} while (gethostname(hostname, sz) == -1);

	return hostname;
}

/*
 * Internal Methods
 */

static void
ical_component_free_categories (ICalComponent *item)
{
	if (item->priv) {
		GList *list;

		for (list = item->priv->categories; list != NULL; list = g_list_next (list))
			g_free (list->data);
		g_list_free (item->priv->categories);

		item->priv->categories = NULL;
	}
}

static void
ical_component_copy_categories (ICalComponent *item, GList *list)
{
	ical_component_free_categories (item);

	for (; list != NULL; list = g_list_next (list)) {
		item->priv->categories =
			g_list_append (item->priv->categories,
				       g_strdup (list->data));
	}
}

/*
 * External Methods
 */

#define CHUNK_SIZE (1024)
void
ical_component_set_fresh_uid (ICalComponent *ical)
{
	gchar *string;
	gchar *hostname = NULL;

	struct tm tm;
	time_t t;

	g_return_if_fail (ical != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (ical));

	t = time (NULL);
	localtime_r (&t, &tm);

	/* Get host name */

	hostname = ical_component_get_hostname ();

	string = g_strdup_printf("%04d%02d%02dT%02d%02d%02d-%d-%08x@%s",
				 tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
				 tm.tm_hour, tm.tm_min, tm.tm_sec,
				 getpid (), g_random_int (), hostname);

	g_free (hostname);

	g_object_set (G_OBJECT (ical), "uid", string, NULL);

	g_free (string);
}

void
ical_component_set_fresh_created (ICalComponent *ical)
{
	time_t t;
	struct tm tm;

	g_return_if_fail (ical != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (ical));

	t = time (NULL);
	localtime_r (&t, &tm);

	g_object_set (G_OBJECT (ical), "created", &tm, NULL);
}

void
ical_component_set_fresh_dtstamp (ICalComponent *ical)
{
	time_t t;
	struct tm tm;

	g_return_if_fail (ical != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (ical));

	t = time (NULL);
	localtime_r (&t, &tm);

	g_object_set (G_OBJECT (ical), "dtstamp", &tm, NULL);
}

void
ical_component_set_default_organizer (ICalComponent *ical)
{
	char *env;
	gchar *string;

	g_return_if_fail (ical != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (ical));

	/* Determine user's e-mail address */

	env = getenv("EMAIL");
	if (env && env[0] != '\0')
		string = g_strdup_printf ("mailto:%s", env);
	else {
		gchar *hostname;

		hostname = ical_component_get_hostname ();
		string = g_strdup_printf ("mailto:%s@%s",
					  g_get_user_name (),
					  hostname);
		g_free (hostname);
	}

	g_object_set (G_OBJECT (ical), "organizer", string, NULL);

	g_free (string);
}

void
ical_component_set_from_calobj (ICalComponent *item, iCalObject *calobj)
{
	ICalComponentPriv *priv;
	GList *list;

	struct tm dtend_tm, dtstart_tm;
	struct tm created_tm, dtstamp_tm, last_mod_tm;

	ICalStatus status;

	g_return_if_fail (item != NULL);
	g_return_if_fail (ICAL_IS_COMPONENT (item));
	g_return_if_fail (calobj != NULL);

	priv = item->priv;

	localtime_r (&calobj->dtend,    &dtend_tm);
	localtime_r (&calobj->dtstart,  &dtstart_tm);
	localtime_r (&calobj->created,  &created_tm);
	localtime_r (&calobj->dtstamp,  &dtstamp_tm);
	localtime_r (&calobj->last_mod, &last_mod_tm);

	/* Determine status */

	status = ICAL_EVENT_CANCELLED;
	if (calobj->status) {
		if (!strcmp (calobj->status, "TENTATIVE"))
			status = ICAL_EVENT_TENTATIVE;
		else if (!strcmp (calobj->status, "CONFIRMED"))
			status = ICAL_EVENT_CONFIRMED;
		else if (!strcmp (calobj->status, "NEEDS-ACTION") ||
			 !strcmp (calobj->status, "NEEDS ACTION"))
			status = ICAL_TODO_NEEDS_ACTION;
		else if (!strcmp (calobj->status, "COMPLETED"))
			status = ICAL_TODO_COMPLETED;
		else if (!strcmp (calobj->status, "IN-PROCESS") ||
			 !strcmp (calobj->status, "IN PROCESS"))
			status = ICAL_TODO_IN_PROCESS;
		else if (!strcmp (calobj->status, "DRAFT"))
			status = ICAL_JOURNAL_DRAFT;
		else if (!strcmp (calobj->status, "FINAL"))
			status = ICAL_JOURNAL_FINAL;
		else {
			if (strcmp (calobj->status, "CANCELLED"))
				g_warning ("Wrong ICalendar status field, setting to CANCELED.");
		}
	}

	g_object_set (G_OBJECT (item),
		      "class",		calobj->class         ? calobj->class     : NULL,
		      "description",	calobj->comment       ? calobj->comment   : NULL,
		      "priority",	calobj->priority <= 9 ? calobj->priority  : 0,
		      "summary",	calobj->summary       ? calobj->summary   : NULL,

		      "dtend",		calobj->dtend   >= 0  ? &dtend_tm         : NULL,
		      "due",		calobj->dtend   >= 0  ? &dtend_tm         : NULL, /* calobj uses dtend for due date */
		      "dtstart",	calobj->dtstart >= 0  ? &dtstart_tm       : NULL,
		      "transp",		calobj->transp == ICAL_TRANSPARENT,

		      "organizer",	calobj->organizer     ? calobj->organizer : NULL,
		      "uid",		calobj->uid           ? calobj->uid       : NULL,

		      "created",	calobj->created  >= 0 ? &created_tm       : NULL,
		      "dtstamp",	calobj->dtstamp  >= 0 ? &dtstamp_tm       : NULL,
		      "last-modified",	calobj->last_mod >= 0 ? &last_mod_tm      : NULL,
		      "sequence",	calobj->seq      >= 0 ? calobj->seq       : 0,

		      NULL);

	/* Set categories */

	ical_component_free_categories (item);
	for (list = calobj->categories; list != NULL; list = g_list_next (list)) {
		priv->categories =
			g_list_append (priv->categories, g_strdup (list->data));
	}
}

iCalObject *
ical_component_get_calobj (ICalComponent *item)
{
	ICalComponentPriv *priv;
	iCalObject *ical;
	GList *list;

	struct tm dtend_tm, dtstart_tm;
	struct tm created_tm, dtstamp_tm, last_mod_tm;

	ICalStatus status;

	gboolean transparent;

	g_return_val_if_fail (item != NULL, NULL);
	g_return_val_if_fail (ICAL_IS_COMPONENT (item), NULL);

	priv = item->priv;

	ical = ical_object_new ();

	ical->type = ICAL_TODO; /* FIXME: this needs to go away, once we have other sub-classes as well */

	g_object_get (G_OBJECT (item),
		      "class",		&ical->class,
		      "description",	&ical->comment,
		      "priority",	&ical->priority,
		      "status",		&status,
		      "summary",	&ical->summary,

		      "dtend",		&dtend_tm,
		      "due",		&dtend_tm, /* calobj uses dtend for due date */
		      "dtstart",	&dtstart_tm,
		      "transp",		&transparent,

		      "organizer",	&ical->organizer,
		      "uid",		&ical->uid,

		      "created",	&created_tm,
		      "dtstamp",	&dtstamp_tm,
		      "last-modified",	&last_mod_tm,

		      NULL);

	ical->dtend    = mktime (&dtend_tm);
	ical->dtstart  = mktime (&dtstart_tm);
	ical->created  = mktime (&created_tm);
	ical->dtstamp  = mktime (&dtstamp_tm);
	ical->last_mod = mktime (&last_mod_tm);

	/* Set status */

	switch (status) {
	case ICAL_EVENT_CANCELLED:
		ical->status = g_strdup ("CANCELLED");
		break;
	case ICAL_EVENT_TENTATIVE:
		ical->status = g_strdup ("TENTATIVE");
		break;
	case ICAL_EVENT_CONFIRMED:
		ical->status = g_strdup ("CONFIRMED");
		break;
	case ICAL_TODO_NEEDS_ACTION:
		ical->status = g_strdup ("NEEDS-ACTION");
		break;
	case ICAL_TODO_COMPLETED:
		ical->status = g_strdup ("COMPLETED");
		break;
	case ICAL_TODO_IN_PROCESS:
		ical->status = g_strdup ("IN-PROCESS");
		break;
	case ICAL_JOURNAL_DRAFT:
		ical->status = g_strdup ("DRAFT");
		break;
	case ICAL_JOURNAL_FINAL:
		ical->status = g_strdup ("FINAL");
		break;
	}

	/* Set date transparency */

	ical->transp = priv->transp ? ICAL_TRANSPARENT : ICAL_OPAQUE;

	/* Set categories */

	ical->categories = NULL;
	for (list = priv->categories; list != NULL; list = g_list_next (list)) {
		ical->categories =
			g_list_append (ical->categories, g_strdup (list->data));
	}

	return ical;
}

static void
ical_component_add_category_to_string (gpointer data, gpointer user_data)
{
	GString     *string;
	const gchar *text;

	g_return_if_fail (data != NULL);
	g_return_if_fail (user_data != NULL);

	text   = (const gchar *) data;
	string = (GString *)     user_data;

	if (string->str[0] != '\0')
		g_string_append (string, ", ");
	g_string_append (string, text);
}

gchar *
ical_component_get_categories_as_string (ICalComponent *item)
{
	GString *string;

	g_return_val_if_fail (item != NULL, NULL);
	g_return_val_if_fail (ICAL_IS_COMPONENT (item), NULL);

	string = g_string_new ("");

	g_list_foreach (item->priv->categories,
			ical_component_add_category_to_string,
			string);

	return g_string_free (string, FALSE);
}
