/* GnomeCard - a graphical contact manager.
 *
 * canvas.c: This file is part of GnomeCard.
 *
 * $Id: canvas.c,v 1.25.4.11 2002/07/19 09:14:37 srittau Exp $
 * 
 * Copyright (C) 1999 The Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Changed in 2002 by Sebastian Rittau <srittau@jroger.in-berlin.de>
 * for the GNOME 2 port.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <gnome.h>

#include "card.h"
#include "dialog.h"
#include "gnomecard.h"
#include "canvas.h"
#include "dnd.h"
#include "my.h"

#define CARDNAME_FONT	"Sans 18"
#define TITLE_FONT	"Monospace 14"
#define COMMENT_FONT	"Monospace 10"
#define CANVAS_FONT	"Monospace 12"
#define ADDR_FONT	"Monospace 10"

#define CANVAS_WIDTH 225
#define CANVAS_HEIGHT 350
#define LIST_SPACING 15.0
#define ADDR_SPACING 11.0

#define HEADER_BOX_COLOR  "khaki"
#define SUBHEADER_BOX_COLOR ""
#define LABEL_BOX_COLOR "dark khaki"

#define HEADER_TEXT_COLOR "black"
#define LABEL_TEXT_COLOR   "black"
#define BODY_TEXT_COLOR    "black"

GtkWidget *gnomecard_canvas;
static GnomeCanvasItem *cardname, *canvastext;
static GnomeCanvasItem *cardname_box, *bg_box;


static GtkTargetEntry
drag_types [] = {
	{ "text/x-vcard",          0, TARGET_VCARD },
	{ "x-application/x-email", 0, TARGET_EMAIL },
	{ "text/plain",            0, TARGET_TEXT },
};


static void
drag_data_get (GtkWidget        *widget,
               GdkDragContext   *context,
               GtkSelectionData *selection_data,
               guint            info,
               guint32          time)
{
	MIMEDirVCard *card;
	char *data = NULL;

	card = (MIMEDirVCard *) g_object_get_data (G_OBJECT (gnomecard_canvas), "card");
	if (!card)
		return;

	g_assert (MIMEDIR_IS_VCARD (card));

	switch (info) {
	case TARGET_TEXT: 
		data = mimedir_vcard_get_as_string (card);
		break;
                
	case TARGET_EMAIL: {
		const gchar *name, *email;

		g_object_get (G_OBJECT (card), "name", &name, "email", &email, NULL);

		if (email && *email) {
			const gchar *name;

			if (name && *name)
				data = g_strdup_printf ("%s <%s>", name, email);
			else
				data = g_strdup_printf("<%s>", email);
		}

		break;
	}
                
	case TARGET_VCARD:
		data = mimedir_vcard_write_to_buffer (card);
		break;
	}

	if (!data)
		return;
        
	gtk_selection_data_set (selection_data, selection_data->target, 8,
				data, strlen (data)+1);

	g_free (data);
}


GtkWidget *
gnomecard_canvas_new(void)
{
	GtkWidget *canvas;
	GnomeCanvasGroup *root;
	gdouble x1, x2, y1, y2;

	canvas = gnomecard_canvas = gnome_canvas_new();

	g_object_set_data (G_OBJECT (canvas), "card", NULL);
	g_object_set_data (G_OBJECT (canvas), "card_signal", GINT_TO_POINTER (0UL));

	/* Drag & Drop */

        gtk_drag_source_set (canvas, GDK_BUTTON1_MASK,
                             drag_types, 3,
                             GDK_ACTION_COPY | GDK_ACTION_DEFAULT);
        g_signal_connect (G_OBJECT (canvas), "drag_data_get",
			  G_CALLBACK (drag_data_get), NULL);

	gtk_widget_pop_colormap();

	gtk_widget_set_size_request (GTK_WIDGET(canvas), 
				     CANVAS_WIDTH, CANVAS_HEIGHT);

	root = GNOME_CANVAS_GROUP (gnome_canvas_root(GNOME_CANVAS(canvas)));

	bg_box = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
			       "x1", 0.0,
			       "y1", 0.0,
			       "x2", (double) (CANVAS_WIDTH - 1),
			       "y2", (double) (CANVAS_HEIGHT - 1),
			       "fill_color", "white",
			       "outline_color", "black",
			       "width_pixels", 1,
			       NULL);

	/* label at top of canvas, contains current cardname */
	cardname_box = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
			       "x1", 5.0,
			       "y1", 5.0,
			       "x2", (double) (CANVAS_WIDTH - 10),
			       "y2", (double) 35,
			       "fill_color", HEADER_BOX_COLOR,
			       "outline_color", HEADER_BOX_COLOR,
			       "width_pixels", 0,
			       NULL);

	cardname = gnome_canvas_item_new (root, gnome_canvas_text_get_type (),
				      "text", "",
				      "x", 6.0, /*CANVAS_WIDTH / 2.0,*/
				      "y", 12.0,
				      "font", CARDNAME_FONT,
				      "anchor", GTK_ANCHOR_NORTH_WEST,
				      "fill_color", HEADER_TEXT_COLOR,
				      NULL);

	canvastext = gnome_canvas_item_new (root, gnome_canvas_text_get_type (),
				      "text", "",
				      "x", 5.0,
				      "y", 45.0,
				      "font", CANVAS_FONT,
				      "anchor", GTK_ANCHOR_NORTH_WEST,
				      "fill_color", BODY_TEXT_COLOR,
				      NULL);
	/* set scrolling region */
	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(root), &x1,&y1,&x2,&y2);
	gnome_canvas_set_scroll_region(GNOME_CANVAS(canvas), x1, y1, x2, y2);

	/* all done */
	return canvas;
}

void
gnomecard_clear_canvas (GnomeCanvas *canvas)
{
	MIMEDirVCard *vcard;

	vcard = g_object_get_data (G_OBJECT (gnomecard_canvas), "card");

	if (vcard) {
		gulong signal;

		signal = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (gnomecard_canvas), "card_signal"));
		g_signal_handler_disconnect (G_OBJECT (vcard), signal);
		g_object_unref (G_OBJECT (vcard));
	}

	g_object_set_data (G_OBJECT (gnomecard_canvas), "card", NULL);
	g_object_set_data (G_OBJECT (gnomecard_canvas), "card_signal", GINT_TO_POINTER (0UL));

	gnome_canvas_item_set (cardname,   "text", "", NULL);
	gnome_canvas_item_set (canvastext, "text", "", NULL);
}


static void
canvas_update (GnomeCanvas *canvas, MIMEDirVCard *card)
{
	GnomeCanvasGroup *root;
	gchar *text;
	gdouble x1, x2, y1, y2;
	gdouble width, height;

	text = mimedir_vcard_get_as_string (card);
	gnome_canvas_item_set (canvastext, "text", text, NULL);
	g_free (text);

	/* card name */
	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(canvastext), 
				     NULL, &y1, &x2, &y2);

	{
		PangoLayout *layout;
		PangoFontDescription *font;
		gchar *str;
		int w;

		g_object_get (G_OBJECT (card), "name", &str, NULL);
		if (!str || *str == '\0')
			str = _("No Card Name");

		font = pango_font_description_from_string (CARDNAME_FONT);
		g_assert (font != NULL);

		layout = gtk_widget_create_pango_layout (gnomecard_canvas, str);
		pango_layout_set_font_description (layout, font);

		pango_layout_get_pixel_size (layout, &w, NULL);
		width = (gdouble) w;

		pango_font_description_free (font);
		g_object_unref (G_OBJECT (layout));

		width = MAX (CANVAS_WIDTH - 5.0, width);
		width = MAX (x2, width);
		gnome_canvas_item_set (cardname, "text", str, NULL);
		gnome_canvas_item_set (cardname_box, "x2", width, NULL);
	}

	width += 4.0;
	height = MAX(CANVAS_HEIGHT - 1, y2 - y1 + 45);
	gnome_canvas_item_set(bg_box, 
			      "x2", width, 
			      "y2", height, 
			      NULL);

	/* set scrolling region */
	root = GNOME_CANVAS_GROUP (gnome_canvas_root (GNOME_CANVAS (gnomecard_canvas)));
	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(root), &x1,&y1,&x2,&y2);
	gnome_canvas_set_scroll_region(GNOME_CANVAS(gnomecard_canvas),
				       x1, y1, x2, y2);
}


void
gnomecard_update_canvas (GnomeCanvas *canvas, MIMEDirVCard *vcard)
{
	MIMEDirVCard *old_card;
	gulong signal;

	g_return_if_fail (vcard != NULL);
	g_return_if_fail (MIMEDIR_IS_VCARD (vcard));

	old_card = g_object_get_data (G_OBJECT (gnomecard_canvas), "card");

	if (old_card) {
		signal = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (gnomecard_canvas), "card_signal"));
		g_signal_handler_disconnect (G_OBJECT (old_card), signal);
		g_object_unref (G_OBJECT (old_card));
	}

	g_object_ref (vcard);
	signal = g_signal_connect_swapped (G_OBJECT (vcard), "changed", G_CALLBACK (canvas_update), canvas);
	g_object_set_data (G_OBJECT (gnomecard_canvas), "card", vcard);
	g_object_set_data (G_OBJECT (gnomecard_canvas), "card_signal", GINT_TO_POINTER (signal));

	canvas_update (canvas, vcard);
}
